from distutils.core import setup
setup(
    name='nepalicalendar',
    packages=['nepalicalendar'],
    version='0.3',
    description='Nepali Calendar Library',
    author='Anup Pokhrel',
    author_email='virtualanup@gmail.com',
    url='https://github.com/nepalicalendar/nepalicalendar-py',
    download_url='https://github.com/nepalicalendar/nepalicalendar-py/tarball/0.3',
    keywords=['calendar', 'nepalicalendar', 'nepcal'],
    classifiers=[],
)
